'Si5351 Driver V1.0
'By Ozpropdev 26th Jan 2016


{{
********************************************
*   SiLabs Si5351 Clock Genreator Driver   *
*                                          *
*  Basic single channel - 2MHz to 150MHz   *
*                                          *
*         By Ozpropdev - Jan 2016          *
********************************************
}}

var

  long  sck,sda

  long  vco,pll
  long  msna_p1,msna_p2,msna_p3
  long  ms0_p1,ms0_p2,ms0_p3
  long  ms0_divby4,r0_div  

  byte  reg[256]


pub set_pins(p1,p2) 

  sda := p1
  sck := p2
  
  dira[sck] :=1
  dira[sda] :=1
  i2c_stop
  

pub set_freq(freq)

  calc(freq)

  i2c_stop
  send_packet(@list1)
  send_packet(@list2)
  send_regs(15,92)
  send_regs(149,170)
  send_reg(183)
  send_packet(@list3)


pri send_reg(r)

    i2c_start
    i2c_out(%1100000_0)
    wait4ack
    i2c_out(r)  'register address
    wait4ack
    i2c_out(reg[r])  'register contents
    wait4ack
    i2c_stop

    
pri send_regs(s,f)|x

    i2c_start
    i2c_out(%1100000_0)
    wait4ack
    i2c_out(s)  'register address
    wait4ack

    repeat x from s to f
      i2c_out(reg[x])  'register contents
      wait4ack
    
    i2c_stop

  
pri send_packet(x)
  
  repeat
  
    if byte[x] == $ff
      quit

    reg[byte[x]] := byte[x+1]
      
    i2c_start
    i2c_out(%1100000_0)
    wait4ack
    i2c_out(byte[x])  'register address
    x++
    wait4ack
    i2c_out(byte[x])  'register contents
    x++
    wait4ack
    i2c_stop

pri i2c_stop

  outa[sck] := 1  
  outa[sda] := 1

pri i2c_start

  outa[sck] := 1  
  outa[sda] := 0
  outa[sck] := 0  

pri i2c_out(b)

  dira[sda] := 1
  
  repeat 8
      outa[sda] := b >> 7
      outa[sck] := 1
      outa[sck] := 0
      b <<= 1
          
pri wait4ack

  dira[sda] := 0   'make pin input
  
  outa[sck] := 1
'  outa[sck] := 1
'  outa[sck] := 1
'  outa[sck] := 1
  
  repeat while ina[sda]

  outa[sck] := 0
  dira[sda] := 1  'restore pin to output
  
pri calc(freq)|z,a,b,c,q,rn

  vco := 750_000_000 / freq
  if vco & 1
     vco++
  
  pll := vco * freq
   
  a := pll / 25_000_000
  b := pll // 25_000_000
  c := 25_000_000
  
  q := get_gcd(b,c)  
  b /= q
  c /= q
  
  z := b * 10000
  z /= c

  msna_p1 := 128 * a + (128*b/c) - 512
  msna_p2 := 128 * b - c * (128*b/c)
  msna_p3 := c

  a := vco
  b~
  c := 1
  ms0_p1 :=   128 * a + (128*b/c) - 512
  ms0_p2 := 128 * b - c * (128*b/c)
  ms0_p3 := c

'**************************************
  rn := 26
  reg[rn++] := (msna_p3 >> 8) & $ff

  reg[rn++] := msna_p3 & $ff  

  reg[rn++] := (msna_p1 >> 16) & 3   

  reg[rn++] := (msna_p1 >> 8) & $ff   

  reg[rn++] := msna_p1  & $ff   

  reg[rn] := (msna_p3 >> 12)  & $f0
  reg[rn++] |= ((msna_p2 >> 16) & $f)   

  reg[rn++] := (msna_p2 >> 8)  & $ff 

  reg[rn++] := msna_p2 & $ff

'********************************
  rn := 42
  reg[rn++] := (ms0_p3 >> 8) & $ff

  reg[rn++] := ms0_p3 & $ff   

  reg[rn] := (ms0_p1 & 3) >> 16
  reg[rn] |= (ms0_divby4 & 3) << 2
  reg[rn++] |=  (r0_div & 7) << 4

    
  reg[rn++] := (ms0_p1 >> 8) & $ff

  reg[rn++] := ms0_p1 & $ff 

  reg[rn] := (ms0_p2 >> 16) & $f
  reg[rn++] |= (ms0_p3 >> 12) & $f0


  reg[rn++] := (ms0_p2 >> 8) & $ff  

  reg[rn++] := ms0_p2 & $ff 
  
  reg[16] := $4f
  reg[183] := $d2
  
  
pri get_gcd(a,b)|t

  'Greatest common denominator (Euchlid's Algorithm)
  
  repeat until b == 0
    t := b
    b := a // b
    a := t
    
  return a

      
dat

list1   byte    3,$ff   'disable outputs
        byte    16,$80  'powerdown drivers
        byte    17,$80
        byte    18,$80
        byte    19,$80
        byte    20,$80
        byte    21,$80
        byte    22,$80
        byte    23,$80
        byte    $ff

list2   byte    15,$00  'PLL Input Source
        byte    16,$4F  'CLK0 Control
        byte    17,$80  'CLK1 Control
        byte    18,$80  'CLK2 Control
        byte    19,$80  'CLK3 Control
        byte    20,$80  'CLK4 Control
        byte    21,$80  'CLK5 Control
        byte    22,$80  'CLK6 Control
        byte    23,$80  'CLK7 Control
        byte    24,$00  'CLK3_0 Disable State
        byte    25,$00  'CLK7_4 Disable State
        byte    $ff
                        
list3   byte    177,$ac 'soft reset
        byte    3,$f8   'enable outputs
        byte    $ff
